from torch.utils.data import Dataset
import os
import shutil
import xml.etree.ElementTree as ET
from PIL import Image
import numpy as np
import torch


class IAMDataset(Dataset):
    def __init__(self, root="", split="train"):
        # split in ["train", "valid", "test"]
        self.root = root
        self.format_dataset()
        self.samples = self.load_set(split)

    def __len__(self):
        return len(self.samples)

    def __getitem__(self, idx):
        sample = self.samples[idx]
        return self.preprocessing(sample["image"]), sample["label"]

    def preprocessing(self, image):
        # Resize and normalize
        h, w = image.shape[:2]
        new_h = 64
        img = np.array(Image.fromarray(image).resize((w, new_h), Image.BILINEAR))
        if len(img.shape) == 2:
            img = np.expand_dims(img, 0)
        img = (img - 237.42) / 44.27
        return torch.tensor(img, dtype=torch.float32)

    def load_set(self, split):
        samples = list()
        with open(os.path.join(self.root, "iam_lines", f"{split}.txt"), "r") as f:
            for line in f.readlines():
                path = os.path.join(self.root, "iam_lines", line.split(" ")[0])
                text = " ".join(line.split(" ")[1:]).strip()
                img = np.array(Image.open(path))
                samples.append({
                    "image": img,
                    "label": text
                })
        return samples

    def format_dataset(self):
        xml_files_path = os.path.join(self.root, "xml")  # Folder containing train.xml, valid.xml and test.xml
        img_path = os.path.join(self.root, "img")  # Content of IAM lines.tgz

        set_names = ["train", "valid", "test"]

        output_folder = os.path.join(self.root, "iam_lines")
        if os.path.isdir(output_folder):
            return

        for set_name in set_names:
            id = 0
            current_folder = os.path.join(output_folder, set_name)
            os.makedirs(current_folder, exist_ok=True)
            xml_path = os.path.join(xml_files_path, "{}.xml".format(set_name))
            xml_root = ET.parse(xml_path).getroot()
            label_file_path = os.path.join(output_folder, "{}.txt".format(set_name))
            with open(label_file_path, "w") as f_out:
                for page in xml_root:
                    name = page.attrib.get("FileName").split("/")[-1].split(".")[0]
                    img_fold_path = os.path.join(img_path, name.split("-")[0], name)
                    img_paths = [os.path.join(img_fold_path, p) for p in sorted(os.listdir(img_fold_path))]
                    for i, line in enumerate(page[2]):
                        label = line.attrib.get("Value")
                        path_for_label = os.path.join(set_name, "{}.png".format(id))
                        new_path = os.path.join(current_folder, "{}.png".format(id))
                        shutil.copy(img_paths[i], new_path)
                        f_out.write("{} {}\n".format(path_for_label, label))
                        id += 1